(ns wanijo.instance.domain
  (:require [clojure.spec.alpha :as spec]
            [wanijo.framework.neo4j :as neo4j]
            [clojure.pprint :as pprint]))

(spec/def ::created-at ::neo4j/date-str)
(spec/def ::updated-at ::neo4j/date-str)
(spec/def ::name (spec/and (complement empty?) string?))

(neo4j/defquery
  findy-by-schema
  "MATCH (i:instance)-[:of]->(s:schema)
   WHERE s.uuid = {uuid}
   RETURN i
   ORDER BY i.updated_at DESC")

(defn find-by-schema! [schema-uuid]
  (map :i
       (neo4j/exec-query!
        findy-by-schema
        {:uuid schema-uuid})))

(neo4j/defquery create-instance
  "MATCH (s:schema {uuid:{schema_uuid}})
   CREATE (i:instance {uuid:{uuid}})-[:of]->(s)
      SET i.name = {name},
          i.created_at = {created_at},
          i.updated_at = {created_at}")

(neo4j/defquery create-property
  "MATCH (i:instance {uuid:{uuid}}),
         (a:attribute {uuid:{attr_uuid}})
   CREATE (p:property {uuid:{prop_uuid}})-[:of]->(i),
          (p)-[:of]->(a)
     SET p.value = {value},
         p.created_at = {created_at}")

(defn create! [schema-uuid instance]
  (let [instance-uuid (neo4j/uuid)
        now (neo4j/now-str)
        instance-tuple [create-instance
                        {:schema_uuid schema-uuid
                         :name (:name instance)
                         :uuid instance-uuid
                         :created_at now}]
        prop-tuples (for [{:keys [attribute value]} (:properties instance)]
                      [create-property
                       {:uuid instance-uuid
                        :attr_uuid (:uuid attribute)
                        :prop_uuid (neo4j/uuid)
                        :value value
                        :created_at now}])]
    (apply neo4j/exec-queries!
           (concat [instance-tuple]
                   prop-tuples))))

(neo4j/defquery find-by-uuid
  "MATCH (i:instance {uuid:{uuid}})
   RETURN i")

(defn find-by-uuid! [uuid]
  (->> (neo4j/exec-query! find-by-uuid
                          {:uuid uuid})
       (map :i)
       first))

(neo4j/defquery find-properties
  "MATCH (i:instance {uuid:{uuid}}),
         (p:property)-[:of]->(i),
         (p)-[:of]->(a:attribute)
   RETURN p, a
    ORDER BY a.name")

(defn find-properties! [uuid]
  (map #(assoc (:p %)
               :attribute
               (:a %))
       (neo4j/exec-query! find-properties
                          {:uuid uuid})))

(neo4j/defquery schema-of
  "MATCH (i:instance {uuid:{uuid}}),
         (i)-[:of]->(s:schema)
   RETURN s")

(defn schema-of [uuid]
  (-> (neo4j/exec-query! schema-of
                         {:uuid uuid})
      first
      (map :s)))
