(ns wanijo.instance.routes
  (:require [compojure.core :refer [defroutes GET POST DELETE]]
            [ring.util.response :as resp]
            [formulare.core :as form]
            [wanijo.instance
             [view :as view]
             [domain :as domain]
             [forms :as forms-inst]]
            [wanijo.framework.routing :refer [register! path]]
            [wanijo.schema.domain :as domain-schema]
            [wanijo.attribute.domain :as domain-attr]))

(defn list! [schema-uuid req]
  (view/list! (domain-schema/find-by-uuid! schema-uuid)
              (domain/find-by-schema! schema-uuid)
              (forms-inst/with-attributes (domain-attr/required! schema-uuid))
              req))

(defn new! [req]
  (let [schema-uuid (get-in req [:params :schema-uuid])
        form-def (forms-inst/with-attributes (domain-attr/required! schema-uuid))]
    (if (form/valid? form-def req)
      (let [form-data (form/form-data form-def req)
            req-attrs (domain-attr/required! schema-uuid)
            instance (forms-inst/form-data->instance form-data
                                                     req-attrs)]
        (domain/create! schema-uuid
                        instance)
        (resp/redirect (path :instance-list
                             (:params req))))
      (list! schema-uuid req))))

(defn show! [uuid req]
  (let [instance (assoc (domain/find-by-uuid! uuid)
                        :properties
                        (domain/find-properties! uuid))]
    (view/show! instance req)))

(defroutes routes
  (GET (register! :instance-list "/instance/list/:schema-uuid")
       [schema-uuid :as req]
       (list! schema-uuid req))
  (POST (register! :instance-new "/instance/new") []
        new!)
  (GET (register! :instance-show "/instance/:uuid")
       [uuid :as req]
       (show! uuid req)))
